﻿using System;
using System.ComponentModel.DataAnnotations;

namespace WCS.Entity
{
    /// <summary>
    /// WCS任务类型
    /// </summary>
    public class WCS_TASK : OBJ
    {
        /// <summary>
        /// 生产编码
        /// </summary>
        [StringLength(50)]
        public string BARCODE { get; set; }

        /// <summary>
        /// 任务类型
        /// </summary>
        public TaskType TYPE { get; set; }

        /// <summary>
        /// 任务状态
        /// </summary>
        public TaskStatus STATUS { get; set; }

        /// <summary>
        /// 起始地址
        /// </summary>
        [StringLength(20)]
        public string ADDRFROM { get; set; }

        /// <summary>
        /// 目标地址
        /// </summary>
        [StringLength(20)]
        public string ADDRTO { get; set; }

        /// <summary>
        /// 下一个地址
        /// </summary>
        [StringLength(20)]
        public string ADDRNEXT { get; set; }

        /// <summary>
        /// 巷道
        /// </summary>
        [StringLength(10)]
        public string TUNNEL { get; set; }

        /// <summary>
        ///
        /// </summary>
        public int FLOOR { get; set; }

        /// <summary>
        /// 设备
        /// </summary>
        [StringLength(20)]
        public string DEVICE { get; set; }

        /// <summary>
        /// 堆垛机放货站台
        /// </summary>
        [StringLength(20)]
        public string SRMSTATION { get; set; }

        /// <summary>
        ///创建时间
        /// </summary>
        public DateTime CREATETIME { get; set; } = DateTime.Now;

        /// <summary>
        /// 开始时间
        /// </summary>
        public DateTime? STARTTIME { get; set; }

        /// <summary>
        /// 结束时间
        /// </summary>
        public DateTime? ENDTIME { get; set; }

        /// <summary>
        /// 单号 美纹中组盘时使用
        /// </summary>
        public int DOCID { get; set; }

        /// <summary>
        /// 托盘类型1-4
        /// </summary>
        public short PALLETTYPE { get; set; }

        /// <summary>
        /// 产线1-9
        /// </summary>
        public short PRODLINE { get; set; }

        /// <summary>
        /// 满盘数量
        /// </summary>
        public short FULLQTY { get; set; }

        /// <summary>
        /// 高度
        /// </summary>
        public short HEIGHT { get; set; }

        /// <summary>
        /// WMS任务号
        /// </summary>
        public int WMSTASK { get; set; }

        /// <summary>
        /// 任务组ID
        /// </summary>
        public string TaskGroupKey { get; set; }

        /// <summary>
        /// 更新状态
        /// </summary>
        public TaskStatus UPLOADED { get; set; }

        /// <summary>
        /// AGV任务号
        /// </summary>
        public int AgvTask { get; set; }

        /// <summary>
        /// 优先级
        /// </summary>
        public int Priority { get; set; }

        /// <summary>
        /// 手动处理备注
        /// </summary>
        public string ManualRemarks { get; set; }

        /// <summary>
        /// 产品长度
        /// </summary>
        public decimal Length { get; set; }

        /// <summary>
        /// 产品编码
        /// </summary>
        public string MaterialCode { get; set; }
    }

    public enum TaskType
    {
        组盘 = 0,
        入库 = 1,
        出库 = 2,
        移库 = 3,
        输送 = 5,
    }

    public enum TaskStatus
    {
        新建 = 0,
        执行中 = 2,
        入库交管结束 = 3,
        堆垛机执行 = 5,
        堆垛机完成 = 6,
        出库RGV执行 = 9,
        已完成 = 99,
        已取消 = 106
    }
}